@@ -88,17 +88,14 @@ impl Chunk {
         if cores == 1 {
             cores
         } else if complexity == Complexity::Small {
-            if array_length < 500000 {
-                1
-            } else if cores >= 2 { 
-                2 
-            } else { 
-                1 
-            }
+            // Recent tests seem to indicate that even for large arrays it never makes
+            // sense to spawn threads for trivial instructions. A single CPU core with SIMD
+            // is already fast enough to occupy the max memory bandwidth
+            1
         } else if complexity == Complexity::Medium {
-            if array_length < 10000 {
+            if array_length < 50000 {
                 1
-            } else if array_length < 50000 {
+            } else if array_length < 100000 {
                 if cores >= 2 { 2 } else { 1 }
             } else {
                 cores
