@@ -6,6 +6,7 @@ use multicore_support::*;
 use super::super::{array_to_complex_mut, VoidResult, Buffer, Owner, ErrorReason, NumberSpace,
                    Domain, ResizeOps, DspVec, Vector, ToSliceMut, MetaData};
 
+/// This trait allows to reorganize the data by changing positions of the individual elements.
 pub trait ReorganizeDataOps<T>
     where T: RealNumber
 {
@@ -37,6 +38,8 @@ pub trait ReorganizeDataOps<T>
     fn swap_halves(&mut self);
 }
 
+/// This trait allows to reorganize the data by changing positions of the individual elements.
+/// Deprecated since it requires a buffer which is never used since there would be no beneifit in using it.
 #[deprecated(since="0.4.1", note="please use `ReorganizeDataOps` instead which offers the same performance without a buffer")]
 pub trait ReorganizeDataOpsBuffered<S, T>
     where T: RealNumber,
@@ -72,12 +75,14 @@ pub enum PaddingOption {
     Center,
 }
 
+/// A trait to insert zeros into the data at some specified positions.
 pub trait InsertZerosOps<T>
     where T: RealNumber
 {
     /// Appends zeros add the end of the vector until the vector has the size given
     /// in the points argument.
-    /// If `points` smaller than the `self.len()` then this operation won't do anything.
+    /// If `points` smaller than the `self.len()` then this operation won't do anything, however
+    /// in future it will raise an error.
     ///
     /// Note: Each point is two floating point numbers if the vector is complex.
     /// Note2: Adding zeros to the signal changes its power. If this function is used to
@@ -120,13 +125,16 @@ pub trait InsertZerosOps<T>
     fn zero_interleave(&mut self, factor: u32) -> VoidResult;
 }
 
+/// A trait to insert zeros into the data at some specified positions. A buffer is used
+/// for types which can't be resized and/or to speed up the calculation.
 pub trait InsertZerosOpsBuffered<S, T>
     where T: RealNumber,
           S: ToSliceMut<T>
 {
     /// Appends zeros add the end of the vector until the vector has the size given in the
     /// points argument.
-    /// If `points` smaller than the `self.len()` then this operation won't do anything.
+    /// If `points` smaller than the `self.len()` then this operation won't do anything, however
+    /// in future it will raise an error
     ///
     /// Note: Each point is two floating point numbers if the vector is complex.
     /// Note2: Adding zeros to the signal changes its power. If this function is used to
@@ -172,7 +180,7 @@ pub trait InsertZerosOpsBuffered<S, T>
     fn zero_interleave_b<B>(&mut self, buffer: &mut B, factor: u32) where B: Buffer<S, T>;
 }
 
-
+/// Splits the data into several smaller pieces of equal size.
 pub trait SplitOps {
     /// Splits the vector into several smaller vectors. `self.len()` must be dividable by
     /// `targets.len()` without a remainder and this condition must be true too
@@ -197,6 +205,7 @@ pub trait SplitOps {
     fn split_into(&self, targets: &mut [&mut Self]) -> VoidResult;
 }
 
+/// Merges several pieces of equal size into one data chunk.
 pub trait MergeOps {
     /// Merges several vectors into `self`. All vectors must have the same size and
     /// at least one vector must be provided.
@@ -315,7 +324,7 @@ macro_rules! zero_interleave {
 }
 
 impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
-    where S: ToSliceMut<T> + Owner,
+    where S: ToSliceMut<T>,
           T: RealNumber,
           N: NumberSpace,
           D: Domain
@@ -323,7 +332,8 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
     fn zero_pad(&mut self, points: usize, option: PaddingOption) -> VoidResult {
         let len_before = self.len();
         let is_complex = self.is_complex();
-        let len = if is_complex { 2 * points } else { points };
+        let step = if is_complex { 2 } else { 1 };
+        let len = points * step;
         if len <= len_before {
             return Ok(());
         }
@@ -340,9 +350,9 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
                 Ok(())
             }
             PaddingOption::Surround => {
-                let diff = (len - len_before) / if is_complex { 2 } else { 1 };
-                let mut right = diff / 2;
-                let mut left = diff - right;
+                let diff = (len - len_before) / step;
+                let mut right = (diff - 1) / 2;
+                let mut left = diff - right ;
                 if is_complex {
                     right *= 2;
                     left *= 2;
@@ -352,29 +362,37 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
                     let src = &data[0] as *const T;
                     let dest = &mut data[left] as *mut T;
                     ptr::copy(src, dest, len_before);
-                    let dest = &mut data[len - right] as *mut T;
-                    ptr::write_bytes(dest, 0, right);
+                    if right > 0 {
+                        let dest = &mut data[len - right] as *mut T;
+                        ptr::write_bytes(dest, 0, right);
+                    }
                     let dest = &mut data[0] as *mut T;
                     ptr::write_bytes(dest, 0, left);
                 }
                 Ok(())
             }
             PaddingOption::Center => {
-                let mut diff = (len - len_before) / if is_complex { 2 } else { 1 };
-                let mut right = diff / 2;
-                let mut left = diff - right;
+                let points_before = len_before / step;
+                let mut right = (points_before - 1) / 2;
+                let mut left = points_before - right ;
                 if is_complex {
                     right *= 2;
                     left *= 2;
-                    diff *= 2;
+                }
+
+                if right > 0 {
+                    unsafe {
+                        let src = &data[left] as *const T;
+                        let dest = &mut data[len - right] as *mut T;
+                        // src and dest may overload with copy, a unit test covers this case
+                        ptr::copy(src, dest, right);
+                    }
                 }
 
                 unsafe {
-                    let src = &data[left] as *const T;
-                    let dest = &mut data[len - right] as *mut T;
-                    ptr::copy(src, dest, right);
                     let dest = &mut data[left] as *mut T;
-                    ptr::write_bytes(dest, 0, len - diff);
+                    ptr::write_bytes(dest, 0, len - left - right);
+
                 }
                 Ok(())
             }
@@ -454,7 +472,7 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
                 }
                 PaddingOption::Surround => {
                     let diff = (len - len_before) / if is_complex { 2 } else { 1 };
-                    let mut right = diff / 2;
+                    let mut right = (diff - 1) / 2;
                     let mut left = diff - right;
                     if is_complex {
                         right *= 2;
@@ -465,16 +483,18 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
                         let src = &data[0] as *const T;
                         let dest = &mut target[left] as *mut T;
                         ptr::copy(src, dest, len_before);
-                        let dest = &mut target[len - right] as *mut T;
-                        ptr::write_bytes(dest, 0, right);
+                        if right > 0 {
+                            let dest = &mut target[len - right] as *mut T;
+                            ptr::write_bytes(dest, 0, right);
+                        }
                         let dest = &mut target[0] as *mut T;
                         ptr::write_bytes(dest, 0, left);
                     }
                 }
                 PaddingOption::Center => {
                     let mut diff = (len - len_before) / if is_complex { 2 } else { 1 };
-                    let mut right = diff / 2;
-                    let mut left = diff - right;
+                    let mut right = (diff - 1) / 2;
+                    let mut left = diff - right ;
                     if is_complex {
                         right *= 2;
                         left *= 2;
@@ -482,9 +502,12 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
                     }
 
                     unsafe {
-                        let src = &data[left] as *const T;
+                        let src = &data[len_before / 2 + 1] as *const T;
                         let dest = &mut target[len - right] as *mut T;
                         ptr::copy(src, dest, right);
+                        let src = &data[0] as *const T;
+                        let dest = &mut target[0] as *mut T;
+                        ptr::copy(src, dest, left);
                         let dest = &mut target[left] as *mut T;
                         ptr::write_bytes(dest, 0, len - diff);
                     }
@@ -640,6 +663,37 @@ mod tests {
         assert_eq!(&v[..], &expected);
     }
 
+    #[test]
+    fn zero_pad_center_test() {
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
+        v.zero_pad(10, PaddingOption::Center).unwrap();
+        let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 7.0, 8.0, 9.0, 10.0];
+        assert_eq!(&v[..], &expected);
+    }
+
+
+    #[test]
+    fn zero_pad_b_center_test() {
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
+        let mut buffer = SingleBuffer::new();
+        v.zero_pad_b(&mut buffer, 10, PaddingOption::Center);
+        let expected = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+            0.0, 0.0, 7.0, 8.0, 9.0, 10.0];
+        assert_eq!(&v[..], &expected);
+    }
+
+    #[test]
+    fn zero_pad_surround_odd_signal_test() {
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0].to_real_time_vec();
+        v.zero_pad(20, PaddingOption::Surround).unwrap();
+        // The expected result is required so that the convolution theorem holds true
+        // (mul in freq is the same as conv)
+        let expected = [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0,
+                        10.0, 11.0, 0.0, 0.0, 0.0, 0.0];
+        assert_eq!(&v[..], &expected);
+    }
+
     #[test]
     fn zero_pad_b_end_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
@@ -660,6 +714,18 @@ mod tests {
         assert_eq!(&v[..], &expected);
     }
 
+    #[test]
+    fn zero_pad_b_surround_odd_signal_test() {
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0, 12.0].to_complex_time_vec();
+        let mut buffer = SingleBuffer::new();
+        v.zero_pad_b(&mut buffer, 10, PaddingOption::Surround);
+        // The expected result is required so that the convolution theorem holds true
+        // (mul in freq is the same as conv)
+        let expected = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0,
+                        10.0, 11.0, 12.0, 0.0, 0.0];
+        assert_eq!(&v[..], &expected);
+    }
+
     #[test]
     fn zero_pad_on_slice_fail_test() {
         let a: Box<[f64]> = Box::new([1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0]);
@@ -680,6 +746,23 @@ mod tests {
         assert_eq!(&v[..], &expected);
     }
 
+    #[test]
+    fn zero_pad_surround_overlap_test() {
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
+        v.zero_pad(8, PaddingOption::Surround).unwrap();
+        let expected = [0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 0.0, 0.0];
+        assert_eq!(&v[..], &expected);
+    }
+
+    #[test]
+    fn zero_pad_center_overlap_test() {
+        let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0].to_complex_time_vec();
+        v.zero_pad(8, PaddingOption::Center).unwrap();
+        let expected = [ 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.0, 8.0, 9.0, 10.0];
+        assert_eq!(&v[..], &expected);
+    }
+
+
     #[test]
     fn zero_interleave_test() {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0].to_real_time_vec();
