@@ -223,6 +223,8 @@ pub struct DspVec<S, T, N, D>
           N: NumberSpace,
           D: Domain
 {
+    /// The underlying storage. `self.len()` should be called to find out how many
+    /// elements in `data` contain valid data.
     pub data: S,
     delta: T,
     domain: D,
@@ -231,6 +233,16 @@ pub struct DspVec<S, T, N, D>
     multicore_settings: MultiCoreSettings,
 }
 
+/// Holds meta data about a type.
+#[derive(Clone)]
+#[derive(Copy)]
+pub struct TypeMetaData<T, N, D> {
+    delta: T,
+    domain: D,
+    number_space: N,
+    multicore_settings: MultiCoreSettings,
+}
+
 impl<S, T, N, D> fmt::Debug for DspVec<S, T, N, D>
     where S: ToSlice<T>,
           T: RealNumber,
