@@ -4,7 +4,8 @@ use num::{Complex, Zero};
 use std::result;
 use super::{round_len, DataDomain, NumberSpace, Domain, ErrorReason, DspVec, GenDspVec,
             RealTimeVec, RealFreqVec, ComplexTimeVec, ComplexFreqVec, RealData, ComplexData,
-            RealOrComplexData, TimeData, FrequencyData, TimeOrFrequencyData, ToSlice};
+            RealOrComplexData, TimeData, FrequencyData, TimeOrFrequencyData, ToSlice,
+            TypeMetaData, MetaData};
 use multicore_support::MultiCoreSettings;
 use std::convert::From;
 
@@ -21,6 +22,12 @@ pub trait ToDspVector<T>: Sized + ToSlice<T>
     ///
     /// For complex vectors with an odd length the resulting value will have a zero length.
     fn to_gen_dsp_vec(self, is_complex: bool, domain: DataDomain) -> GenDspVec<Self, T>;
+
+    /// Create a new vector from the given meta data. The meta data can be
+    /// retrieved from an existing vector. If no existing vector is available
+    /// then one of the other constructor methods should be used.
+    fn to_dsp_vec<N, D>(self, meta_data: &TypeMetaData<T, N, D>) -> DspVec<Self, T, N, D>
+        where N: NumberSpace, D: Domain;
 }
 
 /// Conversion from a generic data type into a dsp vector with real data.
@@ -103,6 +110,22 @@ impl<T> ToDspVector<T> for Vec<T>
             multicore_settings: MultiCoreSettings::default(),
         }
     }
+
+    fn to_dsp_vec<N, D>(self, meta_data: &TypeMetaData<T, N, D>) -> DspVec<Self, T, N, D>
+        where N: NumberSpace, D: Domain {
+        let mut len = self.len();
+        if len % 2 != 0 && meta_data.is_complex() {
+            len = 0;
+        }
+        DspVec {
+            data: self,
+            delta: meta_data.delta,
+            domain: meta_data.domain.clone(),
+            number_space: meta_data.number_space.clone(),
+            valid_len: len,
+            multicore_settings: meta_data.multicore_settings
+        }
+    }
 }
 
 impl<T> ToRealVector<T> for Vec<T>
@@ -146,7 +169,7 @@ impl<T> ToComplexVector<Vec<T>, T> for Vec<T>
             delta: T::one(),
             domain: TimeData,
             number_space: ComplexData,
-            valid_len: len,
+            valid_len: if len % 2 == 0 { len } else { 0 },
             multicore_settings: MultiCoreSettings::default(),
         }
     }
@@ -159,7 +182,7 @@ impl<T> ToComplexVector<Vec<T>, T> for Vec<T>
             delta: T::one(),
             domain: FrequencyData,
             number_space: ComplexData,
-            valid_len: len,
+            valid_len: if len % 2 == 0 { len } else { 0 },
             multicore_settings: MultiCoreSettings::default(),
         }
     }
@@ -209,6 +232,22 @@ impl<'a, T> ToDspVector<T> for &'a [T]
             multicore_settings: MultiCoreSettings::default(),
         }
     }
+
+    fn to_dsp_vec<N, D>(self, meta_data: &TypeMetaData<T, N, D>) -> DspVec<Self, T, N, D>
+        where N: NumberSpace, D: Domain {
+        let mut len = self.len();
+        if len % 2 != 0 && meta_data.is_complex() {
+            len = 0;
+        }
+        DspVec {
+            data: self,
+            delta: meta_data.delta,
+            domain: meta_data.domain.clone(),
+            number_space: meta_data.number_space.clone(),
+            valid_len: len,
+            multicore_settings: meta_data.multicore_settings
+        }
+    }
 }
 
 impl<'a, T> ToRealVector<T> for &'a [T]
@@ -243,31 +282,25 @@ impl<'a, T> ToComplexVector<&'a [T], T> for &'a [T]
     where T: RealNumber
 {
     fn to_complex_time_vec(self) -> ComplexTimeVec<Self, T> {
-        let mut len = self.len();
-        if len % 2 != 0 {
-            len = 0;
-        }
+        let len = self.len();
         ComplexTimeVec {
             data: self,
             delta: T::one(),
             domain: TimeData,
             number_space: ComplexData,
-            valid_len: len,
+            valid_len: if len % 2 == 0 { len } else { 0 },
             multicore_settings: MultiCoreSettings::default(),
         }
     }
 
     fn to_complex_freq_vec(self) -> ComplexFreqVec<Self, T> {
-        let mut len = self.len();
-        if len % 2 != 0 {
-            len = 0;
-        }
+        let len = self.len();
         ComplexFreqVec {
             data: self,
             delta: T::one(),
             domain: FrequencyData,
             number_space: ComplexData,
-            valid_len: len,
+            valid_len: if len % 2 == 0 { len } else { 0 },
             multicore_settings: MultiCoreSettings::default(),
         }
     }
@@ -320,6 +353,22 @@ impl<'a, T> ToDspVector<T> for &'a mut [T]
             multicore_settings: MultiCoreSettings::default(),
         }
     }
+
+    fn to_dsp_vec<N, D>(self, meta_data: &TypeMetaData<T, N, D>) -> DspVec<Self, T, N, D>
+        where N: NumberSpace, D: Domain {
+        let mut len = self.len();
+        if len % 2 != 0 && meta_data.is_complex() {
+            len = 0;
+        }
+        DspVec {
+            data: self,
+            delta: meta_data.delta,
+            domain: meta_data.domain.clone(),
+            number_space: meta_data.number_space.clone(),
+            valid_len: len,
+            multicore_settings: meta_data.multicore_settings
+        }
+    }
 }
 
 impl<'a, T> ToRealVector<T> for &'a mut [T]
@@ -354,31 +403,25 @@ impl<'a, T> ToComplexVector<&'a mut [T], T> for &'a mut [T]
     where T: RealNumber
 {
     fn to_complex_time_vec(self) -> ComplexTimeVec<Self, T> {
-        let mut len = self.len();
-        if len % 2 != 0 {
-            len = 0;
-        }
+        let len = self.len();
         ComplexTimeVec {
             data: self,
             delta: T::one(),
             domain: TimeData,
             number_space: ComplexData,
-            valid_len: len,
+            valid_len: if len % 2 == 0 { len } else { 0 },
             multicore_settings: MultiCoreSettings::default(),
         }
     }
 
     fn to_complex_freq_vec(self) -> ComplexFreqVec<Self, T> {
-        let mut len = self.len();
-        if len % 2 != 0 {
-            len = 0;
-        }
+        let len = self.len();
         ComplexFreqVec {
             data: self,
             delta: T::one(),
             domain: FrequencyData,
             number_space: ComplexData,
-            valid_len: len,
+            valid_len: if len % 2 == 0 { len } else { 0 },
             multicore_settings: MultiCoreSettings::default(),
         }
     }
@@ -431,6 +474,22 @@ impl<T> ToDspVector<T> for Box<[T]>
             multicore_settings: MultiCoreSettings::default(),
         }
     }
+
+    fn to_dsp_vec<N, D>(self, meta_data: &TypeMetaData<T, N, D>) -> DspVec<Self, T, N, D>
+        where N: NumberSpace, D: Domain {
+        let mut len = self.len();
+        if len % 2 != 0 && meta_data.is_complex() {
+            len = 0;
+        }
+        DspVec {
+            data: self,
+            delta: meta_data.delta,
+            domain: meta_data.domain.clone(),
+            number_space: meta_data.number_space.clone(),
+            valid_len: len,
+            multicore_settings: meta_data.multicore_settings
+        }
+    }
 }
 
 impl<T> ToRealVector<T> for Box<[T]>
