@@ -1,7 +1,7 @@
 //! This module defines the basic vector trait and indexers.
 use RealNumber;
 use super::{array_to_complex, array_to_complex_mut, DspVec, NumberSpace, ComplexNumberSpace,
-            Domain, DataDomain, ToSlice, ToSliceMut, ErrorReason, VoidResult};
+            Domain, DataDomain, ToSlice, ToSliceMut, ErrorReason, VoidResult, TypeMetaData};
 use multicore_support::MultiCoreSettings;
 use std::ops::*;
 use num::complex::Complex;
@@ -69,7 +69,7 @@ pub trait Vector<T>: MetaData + ResizeOps
     /// The number of valid elements in the vector. This can be changed
     /// with the `Resize` trait.
     fn len(&self) -> usize;
-    
+
     /// Indicates whether or not the vector is empty.
     fn is_empty(&self) -> bool;
 
@@ -94,6 +94,27 @@ pub trait Vector<T>: MetaData + ResizeOps
     fn alloc_len(&self) -> usize;
 }
 
+/// Gets the meta data of a type. This can be used to create a new type with the same
+/// meta data.
+/// # Example
+///
+/// ```
+/// use basic_dsp_vector::*;
+/// let vector = vec!(1.0, 2.0, 3.0, 4.0, 5.0, 6.0).to_real_time_vec();
+/// let meta_data = vector.get_meta_data();
+/// let slice = &vector[0..2];
+/// let slice = slice.to_dsp_vec(&meta_data);
+/// assert_eq!(false, slice.is_complex());
+/// ```
+pub trait GetMetaData<T, N, D>
+    where T: RealNumber,
+          N: NumberSpace,
+          D: Domain {
+    /// Gets a copy of the vector meta data. This can be used to create
+    /// new types with the same meta data.
+    fn get_meta_data(&self) -> TypeMetaData<T, N, D>;
+}
+
 impl<S, T, N, D> MetaData for DspVec<S, T, N, D>
     where S: ToSlice<T>,
           T: RealNumber,
@@ -109,6 +130,20 @@ impl<S, T, N, D> MetaData for DspVec<S, T, N, D>
     }
 }
 
+impl<T, N, D> MetaData for TypeMetaData<T, N, D>
+    where T: RealNumber,
+          N: NumberSpace,
+          D: Domain
+{
+    fn domain(&self) -> DataDomain {
+        self.domain.domain()
+    }
+
+    fn is_complex(&self) -> bool {
+        self.number_space.is_complex()
+    }
+}
+
 impl<S, T, N, D> ResizeOps for DspVec<S, T, N, D>
     where S: ToSlice<T>,
           T: RealNumber,
@@ -147,7 +182,7 @@ impl<S, T, N, D> Vector<T> for DspVec<S, T, N, D>
     fn len(&self) -> usize {
         self.valid_len
     }
-    
+
     fn is_empty(&self) -> bool {
         self.valid_len == 0
     }
@@ -169,6 +204,21 @@ impl<S, T, N, D> Vector<T> for DspVec<S, T, N, D>
     }
 }
 
+impl<S, T, N, D> GetMetaData<T, N, D> for DspVec<S, T, N, D>
+    where S: ToSlice<T>,
+          T: RealNumber,
+          N: NumberSpace,
+          D: Domain {
+    fn get_meta_data(&self) -> TypeMetaData<T, N, D> {
+        TypeMetaData {
+            number_space: self.number_space.clone(),
+            domain: self.domain.clone(),
+            delta: self.delta,
+            multicore_settings: self.multicore_settings
+        }
+    }
+}
+
 impl<S, T, N, D> Index<usize> for DspVec<S, T, N, D>
     where S: ToSlice<T>,
           T: RealNumber,
